/*
 * 
 * 19.01.2019
   LAN - Server - Decoder - Feedback.
   Using an Arduino Leonardo.
   Using an Arduino Ethernet shield.

   Circuit:
   Ethernet shield attached to pins 10, 4
   Input pins: 2, 3, 5, 6, 7, 8, 9, 11 
 *
 */

#include <SPI.h>
#include <Ethernet.h>

// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network.
// gateway and subnet are optional:
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 1, 77);
IPAddress myDns(192,168,1, 1);
IPAddress gateway(192, 168, 1, 1);
IPAddress subnet(255, 255, 255, 0);

//  defaults to port 4711
EthernetServer server(4711);

String readString = String();

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const int dAnz = 8;
int dPin[dAnz]       = {2, 3, 5, 6, 7, 8, 9, 11};
int dAdr[dAnz]       = {19, 20, 21, 22, 23, 24, 25, 4096}; 
boolean dstate[dAnz] = {false, false, false, false, false, false, false, false};
unsigned int dtime[dAnz] = {0, 0, 0, 0, 0, 0, 0, 0};

int count = -1;    //Zähler für PIN Abfrage, beginne bei 0
#define timefalling 21  // Verzögerung der Rückmeldung HIGH -> LOW (BELEGT)
#define timerising 10
byte speed = 0;
boolean    bool_f0 = false;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup() {
  //Выставляем управляющие пины на вывод:
  pinMode(10, OUTPUT);
  pinMode(13, OUTPUT);
    // Open serial communications and wait for port to open:
  Serial.begin(115200);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  
  // initialize the ethernet device
  Ethernet.begin(mac, ip, myDns, gateway, subnet);
  // start listening for clients
  server.begin();
 
  Serial.print("Server address:");
  Serial.println(Ethernet.localIP());

  //Initialization of the inputs
  for (int i = 0; i < dAnz; i++) {
    pinMode(dPin[i], INPUT);       //Pin on entrance
    pinMode(dPin[i], INPUT_PULLUP);   //Pullup activate
  }

}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop() {

  EthernetClient client = server.available(); 
   
// wait for a new client:
  if (!client) { return; }
    Serial.println("new client");
  boolean currentLineIsBlank = true;
  while(client.connected()){
    if (client.available()){
      char c = client.read();

      Serial.write(c);

      if (readString.length() < 30) {
        readString += c;
      }

      if (c == '>') {

        Serial.println(" ");

        //--- obrabotka
        char *clientline = &readString[0]; //string -> char

        // Print it out for debugging
        // Serial.println("---------");
        // Serial.print("clientline:");
        // Serial.println(clientline);

        if (strstr(clientline, "sw36") != 0) {
          Serial.println("SW-36");
          if (strstr(clientline, "straight") != 0) {
            Serial.println("on");
          }
          if (strstr(clientline, "turnout") != 0) {
            Serial.println("off");
          }
        }
        readString = "";
      }
      
      // Если добрались до конца строки (т.е. получили символ новой строки),
      // и эта строка – пустая, это значит, что это конец HTTP-запроса.
      // То есть, можно приступать к отправке ответа:
      if (c == '\n' && currentLineIsBlank) {
        // Отсылаем стандартный заголовок для HTTP-ответа:
        client.println("HTTP/1.1 200 OK");
        client.println("Content-Type: text/html");
        client.println("Connection: close");  // после выполнения ответа соединение будет разорвано
        //client.println("Refresh: 5");  // автоматически обновляем страницу каждые 5 секунд
        client.println();
        client.println("<!DOCTYPE HTML>");
        client.println("<html>");
        client.print("analog input ");
        client.print("<br />");
        client.print(" is ");
        client.println("</html>");
        break;
      }
      if (c == '\n') {
        // Начинаем новую строку:
        currentLineIsBlank = true;
      }
      else if (c != '\r') {
        // В текущей строке есть символ:
        currentLineIsBlank = false;
      }
    }

    if (client.connected()) {
      count++;    //nächsten Input Abfragen
      if (count >= dAnz)    //Max Input prüfen
        count = 0;        //beim Ersten beginnen

      if (dstate[count] != digitalRead(dPin[count])) {  //Eingang hat sich geändert?
        //dtime[count]++;
        if (dstate[count] == true){    //falling (high -> low)
        //if ((dstate[count] == true) && (dtime[count] >= timefalling)) {    //falling (high -> low)
          client.print("<fb id=\"fb-");
          client.print(dAdr[count], DEC);
          client.println("\" state=\"true\" fbtype=\"0\"/>"); //aktuellen Status senden - ON
          dstate[count] = !dstate[count];             //Speicher aktualisieren
          // dtime[count] = 0;
        }
        else if (dstate[count] == false){     //rising (low -> high)
          //else if ((dstate[count] == false) && (dtime[count] >= timerising)) {     //rising (low -> high)
          client.print("<fb id=\"fb-");
          client.print(dAdr[count], DEC);
          client.println("\" state=\"false\" fbtype=\"0\"/>"); //aktuellen Status senden - OFF
          dstate[count] = !dstate[count];             //Speicher aktualisieren
          // dtime[count] = 0;     
        }
      } // end digitalRead
    } // end client.connected
  } // end while

  client.stop();
  Serial.println("client disonnected");

} //END loop
